/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: list.h,v 1.30 2002/05/07 22:38:29 weinholt Exp $
 */
#ifndef _LIST_H
#define _LIST_H

typedef struct {
	void **entries;
	int num;
	int sel;
	int start;
	int allocated;
} list;

#define LIST_INITIALIZER { NULL, 0, -1, 0, 0 }
#define list_index(l,i)		((l)->entries)[i]

/* allocate a new list */
list *list_new(void);

/* initialize a list */
void list_initialize(list *);

/* free the list entries only */
void list_free_entries(list *);

/* append an entry to a list */
void list_push(list *, void *entry);

/* prepend an entry to a list */
void list_push_first(list *, void *entry);

/* insert an entry at given position */
void list_insert(list *, void *entry, int pos);

/* remove an entry from a list */
void list_remove_entry(list *, int index);
void list_remove_all(list *);

/* do a sanity check of the internal list members */
void list_check_values(list *, int h);

/* return the selected entry */
void *list_selected(list *);

/* insert an entry into a sorted list */
typedef int (*SFunc) (const void *, const void *);
void list_resort(list * snafu, int pos, SFunc cmp);
void list_insort(list * snafu, void *item, SFunc cmp);

typedef void (*LFunc) (void *data);
void list_foreach(list * snafu, LFunc func);

int list_lookup(const char *needle, list * snafu);
int list_find(list * snafu, void *elem);

extern int SCROLLOFF;

#endif
