/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: parse.h,v 1.39 2002/04/20 15:42:38 weinholt Exp $
 */
#ifndef _PARSE_H
#define _PARSE_H

#include <stdarg.h>

/* symbolic name for xml data; terminates with double zeros */
typedef char *xml_data;

/* parse xml tags */
xml_data xml_parse_tag(char *str);

#define xml_get_type(s) s
char *xml_get_param(xml_data, const char *prop);
unsigned long xml_get_param_int(xml_data, const char *prop);
int xml_param_count(xml_data);

/* translates %2e to . and + to ' ' */
char *http_decode(char *str);

/* removes extra whitespace */
char *trim(char *foo);

/* right-pads a string with spaces, until strlen(foo) == lenght */
void strpad(char *foo, int length);

/* translates quantity to a "human-friendly" string, same format as "df -H"
 * for example: 1231424 1000   10000  10239   10240  1023999 1024000
 * becomes    : "1.1M"  "1000" "9.7k" "9.9k"  " 10k" "999k"  "0.9M"
 * the return value points to a ring buffer and is always 4 chars wide
 * Use at most 5 of theese in a printf-call, or increase the ring buffer size.
 */
char *humanify(unsigned long quantity);

/* This uses a predefined scale. 1 => kilo 2 => Mega etc. */
char *humanify_scale(unsigned long quantity, int scale);

/* translate time to a "human.friendly" string, for example
 * 34s, 5m45s, 23h4m, 1d0h, etc.. */
char *humanify_time(unsigned long secs);

#if !HAVE_STRNDUP
char *strndup(const char *s, size_t size);
#endif

#if !HAVE_STRNLEN
size_t strnlen(const char *s, size_t maxlen);
#endif

/* If all-lowercase, ignore case. Otherwise match case. */
char *stristr(const char *haystack, const char *needle);

/* returns the index of string needle in array haystack, or -1 */
int lookup(const char *needle, const char *haystack[], int items);

/* concatenate buf with a formatted string, buf is `max' bytes in size */
int strncatf(char *buf, int max, const char *fmt, ...)
	__attribute__ ((format(printf, 3, 4)));

/* vsprintf to an malloc'ated string */
char *vsprintf_alloc(const char *fmt, va_list ap);
char *sprintf_alloc(const char *fmt, ...)
	__attribute__ ((format(printf, 1, 2)));

void insert_at_ratio(char *buf, char *ins, float ratio, int max);

/* Remove the word at the given position */
char *remove_word(char *foo, int *pos);

#endif
