/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: search.h,v 1.21 2002/06/11 22:43:26 weinholt Exp $
 */
#ifndef _SEARCH_H
#define _SEARCH_H

#include <time.h>
#include "tree.h"
#include "parse.h"
#include "ui_draw.h"
#include "gift.h"
#include "poll.h"

typedef struct {
	rendering *methods;			/* How to draw this item, must be first */
	char *user;					/* The users that have this file */
	char *href;					/* The references to that files */
	int availability;			/* FIXME: hack, remove */
	char *pretty;
} subhit;

typedef struct {
	rendering *methods;			/* How to draw this item, must be first */
	list sources;				/* list of subhits */
	char *hash;
	unsigned long int filesize;
	char *filename;
	char *directory;
	char *pretty;
	unsigned downloading:1;
	unsigned expanded:1;
} hit;

extern rendering hit_methods, subhit_methods;

typedef struct {
	gift_conn *id;
	char *search_term;
	int realm;
	tree hits;
	tick_t date;
	unsigned dirty:1;
} query;

extern list queries;

/* this needs to be called upon startup */
void gift_search_init(void);

/* this needs to be called upon exit */
void gift_search_cleanup(void);

/* start a text query, storing results in main_scr.hits and calling
 * incoming_search_item whenever new hits are received */
int gift_query_start(char *searchstr);

/* start a general query, call fn whenever new hits are received */
gift_conn *gift_search_start(char *searchstr, callback_func fn, void *data);
xml_data gift_search_parse(xml_data line);

/* stop the search associated with id */
void gift_search_stop(gift_conn ** id);

/* stop text query, NULL stops all */
void gift_query_stop(query * q);

/* remove all hits from a query */
void gift_query_clear(query * q);

/* free hit list */
void gift_hits_free(query * q);

/* extract parts of source string */
char *extract_filename_alloc(const char *href);
char *extract_directory_alloc(const char *href);

char *change_sort_method(int direction);

/* Mark hits with this file as currently downloading */
void hit_download_mark(const char *hash, unsigned long size);

void hit_destroy(hit *);
void subhit_destroy(subhit *);

#endif
