/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: stats.c,v 1.11 2002/06/06 16:23:09 chnix Exp $
 */
#include "giftcurs.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "gift.h"
#include "parse.h"
#include "poll.h"
#include "stats.h"

#define STATS_INTERVAL SECS(5)

gift_stat stats;

static enum { STAT_BUSY, STAT_WAITING, STAT_NONE, STAT_FINISH } stat_state = STAT_NONE;

static int gift_stat_recheck(void *data);
static void stat_handle_line(gift_conn * id, xml_data item, void *data);

void gift_stat_update(void (*callback) (void))
{
	if (stat_state != STAT_NONE)
		return;
	memset(&stats, 0, sizeof stats);
	/* if we are not connected, gift_printf will call this function again
	 * if it can connect. To prevent this, we set stat_state to BUSY
	 */
	stat_state = STAT_BUSY;
	if (gift_printf("<stats/>\n") < 0) {
		/* connection failed, set it back to NONE */
		stat_state = STAT_NONE;
		return;
	}
	gift_event(stat_handle_line, callback);
}

static int gift_stat_recheck(void *data)
{
	assert(stat_state == STAT_WAITING);
	stat_state = STAT_NONE;
	gift_stat_update(data);
	return 0;
}

void gift_stat_stop(void)
{
	if (stat_state == STAT_WAITING)
		poll_del_timer(gift_stat_recheck);
	stat_state = STAT_FINISH;
	memset(&stats, 0, sizeof stats);
}

void stat_handle_line(gift_conn * id, xml_data item, void *data)
{
	unsigned long size, files;
	char *plugin;

	/* <stats id=4 protocol=local files= size=/> (for your local shares)
	 * <stats id=4 protocol=OpenFT status=Online files=58943 size=1936 connections=10/>
	 * <stats id=4 protocol=FastTrack status=Offline files=0 size=0 connections=0/>
	 * <stats id=4/>
	 */

	if (stat_state == STAT_FINISH)
		return;
	assert(stat_state == STAT_BUSY);

	if (!item) {
		void (*callback) (void) = data;

		poll_add_timer(STATS_INTERVAL, gift_stat_recheck, callback);
		stat_state = STAT_WAITING;
		callback();
		return;
	}
	files = xml_get_param_int(item, "files");
	size = xml_get_param_int(item, "size");
	plugin = xml_get_param(item, "protocol");
	if (plugin && strcasecmp(plugin, "local")) {
		char *status = xml_get_param(item, "status");

		if (!status || !strcasecmp(status, "online"))
			stats.protos_up++;
		stats.protos++;
		stats.files += files;
		stats.gigs += size;
		stats.nodes += xml_get_param_int(item, "connections");
		stats.users += xml_get_param_int(item, "users");
	} else {
		stats.own_files = files;
		stats.own_gigs = size;
	}
}
