/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: transfer.h,v 1.25 2002/05/20 23:16:46 chnix Exp $
 */
#ifndef _TRANSFER_H
#define _TRANSFER_H

#include "tree.h"
#include "gift.h"
#include "parse.h"
#include "poll.h"

enum {
	TRANSFER_UPLOAD,
	TRANSFER_DOWNLOAD
};

typedef struct {
	rendering *methods;			/* How to display data. MUST be first item */
	char *user;
	char *href;
	unsigned long start;
	unsigned long transmit;
	unsigned long total;
	char *status;
	char *pretty;
} source;

typedef struct {
	rendering *methods;			/* How to display data. MUST be first item */
	gift_conn *id;				/* id to write url's to giFT and get status */
	char *filename;				/* the filename, or NULL if we don't know it yet */
	unsigned long filesize, transferred, bandwidth;
	unsigned long start_size, time;
	unsigned long last_trans, last_time;
	char *pretty;
	unsigned paused:1;
	char *hash;
	list sourcen;				/* download: all the sources we've told giFT about */
	/* upload: all the destinations we send this file to */
	unsigned autodestroy:1;
	unsigned type:1;
	unsigned expanded:1;
	union {
		/* Download specific variables */
		struct {
			gift_conn *search_id;	/* id receiving search results from giFT */
			tick_t latest_search;	/* when did we search for sources? */
		} d;
		/* Upload specific variables */
		struct {
		} u;
	} specific;
} transfer;

void source_destroy(source *);
void source_touch(source *);

transfer *transfer_find(tree * tr, char *hash, unsigned long size);
transfer *transfer_find_dead(tree * tr, char *hash, unsigned long size);
void transfer_destroy(transfer *);
void transfer_touch(transfer *);
void transfer_detach(transfer *);
transfer *transfer_new(int type, char *name, unsigned long size);
void transfer_cancel(transfer *);
void transfer_reset(transfer *);
void transfer_suspend(transfer *);
int transfers_calculate(void);

extern int autodestroy_all;
extern rendering transfer_methods, source_methods;

int transfer_alive(transfer *);

char *transfer_status(transfer *);
char *source_status(source *);

/* NULL in a field act as a wildcard */
source *lookup_source(transfer * t, const char *user, const char *href);
void chunk_parse(transfer * t, xml_data buf);

char *pretty_transfer_original(transfer *);
char *pretty_source_original(transfer *, source *);

char *pretty_transfer_jasta(transfer *);
char *pretty_source_jasta(transfer *, source *);

#endif
