/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: tree.h,v 1.3 2002/04/29 19:28:25 weinholt Exp $
 */
#ifndef _TREE_H
#define _TREE_H

#include "list.h"

/* The virtual functions all tree items must implement. The first thing
 * in such a struct should be a pointer its virtual functions */

typedef struct {
	char *(*pretty_line) (void *);	/* return the line to be drawn */
	list *(*expansion) (void *);	/* return subtree or NULL */
	void (*destroy) (void *);	/* free all data used by this */
	void (*touch) (void *);		/* mark this as changed */
} rendering;

/* A tree is made up of a list of structs implementing the above
 * 'rendering' methods. the 'expansion' method returns the subtree
 * of that item. For displaying purposes, an 'flat' list is also
 * carried around. It can be updated to reflect the tree with the
 * function 'tree_flatten'
 */
typedef struct {
	list top;
	list flat;
	rendering *top_methods;		/* first-level items type */
} tree;

#define tree_isempty(t) ((t)->top.num ? 0 : 1)
#define TREE_INITIALIZER(type) { LIST_INITIALIZER, LIST_INITIALIZER, &type##_methods }

void tree_flatten(tree * t);
int tree_top_sel(tree * t);
void *tree_top_selected(tree * t);
void *tree_find_parent(tree * t, void *item);

void tree_initialize(tree * t, rendering * r);
void tree_destroy_all(tree * t);
void list_destroy_all(list * t);
void tree_touch_all(tree * t);
void list_touch_all(list * t);
void tree_sort(tree * t, SFunc cmp);
void tree_select_top(tree * t, int i);
void tree_select_item(tree * t, void *item);
void tree_remove_item(tree * t, void *item);
void tree_push(tree * t, void *item);
void tree_push_first(tree * t, void *item);
void list_filter(list * t, int (*keep) (void *));
void tree_filter(tree * t, int (*keep) (void *));

#endif
