/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001-2002 Gran Weinholt <weinholt@linux.nu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: upload.c,v 1.58 2002/06/19 22:36:23 weinholt Exp $
 */
#include "giftcurs.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#define __USE_GNU
#include <string.h>

#include "transfer.h"
#include "screen.h"
#include "gift.h"
#include "search.h"
#include "upload.h"
#include "parse.h"
#include "xcommon.h"
#include "ui_transfer.h"

tree uploads = TREE_INITIALIZER(transfer);

#define update_ui() transfer_screen_update_uploads_if_focus()

void upload_forget(transfer * t)
{
	tree_remove_item(&uploads, t);
	transfer_destroy(t);
}

static void ul_status_handler(gift_conn * id, xml_data buf, void *data);

void on_new_upload(xml_data buf)
{
	/* <transfer id="2" action="upload" size="3035" user="127.0.0.1" href="OpenFT://127.0.0.1/home/chn/.giFT/OpenFT/nodepage.html"/> */
	char *href, *hash, *user;
	unsigned long size;
	transfer *entry = NULL;
	source *dest;

	if (!(href = xml_get_param(buf, "href"))) {
		DEBUG("Expected an href attribute.");
		return;
	}

	if (!(user = xml_get_param(buf, "user"))) {
		DEBUG("Expected a user attribute.");
		return;
	}

	if (!(size = xml_get_param_int(buf, "size"))) {
		DEBUG("Expected a size attribute.");
		return;
	}

	hash = xml_get_param(buf, "hash");
	if (hash && !*hash)
		hash = NULL;

	if (hash)
		entry = transfer_find_dead(&uploads, hash, size);

	if (!entry) {
		entry = transfer_new(TRANSFER_UPLOAD, extract_filename_alloc(href), size);
		if (hash)
			entry->hash = strdup(hash);
		/* FIXME: This is protocol dependant */
		/* If it's an OpenFT upload and it lacks the hash, autodestroy it
		   when it's done. */
		if (!strncmp(href, "OpenFT", 6) && !hash)
			entry->autodestroy = 1;
		tree_push_first(&uploads, entry);
	}
	assert(entry->sourcen.num == 0);
	entry->id = gift_event(ul_status_handler, entry);

	dest = NEW(source);
	dest->user = strdup(user);
	dest->href = strdup(href);
	list_push(&entry->sourcen, dest);

	transfer_touch(entry);
	update_ui();
}

/* Callback for incoming data */
static void ul_status_handler(gift_conn * id, xml_data buf, void *data)
{
	/*
	 * <transfer id="2" transmit="3035" total="3035"/>
	 * <transfer id="2"/>
	 */
	transfer *entry = data;

	if (!buf) {
		entry->id = NULL;
		upload_detach(entry);
		if (autodestroy_all || entry->autodestroy)
			upload_forget(entry);
		update_ui();
		return;
	}

	if (!strcmp("chunk", xml_get_type(buf))) {
		chunk_parse(entry, buf);
		return;
	}

	entry->paused = xml_get_param_int(buf, "paused");

	entry->transferred = xml_get_param_int(buf, "transmit");
	if (entry->start_size == (unsigned long) -1)
		entry->start_size = entry->transferred;

	entry->time++;
}

void upload_detach(transfer * t)
{
	gift_unregister(t->id);
	t->id = NULL;
	list_destroy_all(&t->sourcen);
}
